@@ -1,6 +1,12 @@
+#ifdef NXAGENT_UPGRADE
+
+#include "X/NXdixfonts.c"
+
+#else
+
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2004 NoMachine, http://www.nomachine.com.          */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
 /* NXAGENT, NX protocol compression and NX extensions to this software    */
 /* are copyright of NoMachine. Redistribution and use of the present      */
@@ -9,12 +15,13 @@
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
 /**************************************************************************/
 
+/* $XFree86: xc/programs/Xserver/dix/dixfonts.c,v 3.27 2003/02/15 03:47:05 dawes Exp $ */
 /************************************************************************
 Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts.
 
@@ -38,6 +45,8 @@ SOFTWARE.
 
 ************************************************************************/
 
+/* $Xorg: dixfonts.c,v 1.4 2000/08/17 19:48:18 cpqbld Exp $ */
+
 #define NEED_REPLIES
 #include "X.h"
 #include "Xmd.h"
@@ -59,9 +68,70 @@ SOFTWARE.
 #include	<stdio.h>
 #endif
 
-#ifdef NXAGENT_FONTMATCH
-# include "Agent.h"
-# include "Font.h"
+#include "Agent.h"
+#include "Font.h"
+
+#ifndef NX_TRANS_SOCKET
+
+#define NX_TRANS_SOCKET
+
+#endif
+
+#ifdef NX_TRANS_SOCKET
+
+char _NXFontPath[1024];
+
+/*
+ * Override the default font path and make
+ * it configurable at run time, based on
+ * the NX_FONT environment.
+ */
+
+static const char *_NXGetFontPath(const char *path)
+{
+  const char *fontEnv;
+
+    /*
+     * Check the environment only once.
+     */
+
+    if (*_NXFontPath != '\0')
+    {
+        return _NXFontPath;
+    }
+
+    fontEnv = getenv("NX_FONT");
+
+    if (fontEnv != NULL && *fontEnv != '\0')
+    {
+        if (strlen(fontEnv) + 1 > 1024)
+        {
+#ifdef NX_TRANS_TEST
+            fprintf(stderr, "_NXGetFontPath: WARNING! Maximum length of font path exceeded.\n");
+#endif
+            goto _NXGetFontPathError;
+        }
+
+        strcpy(_NXFontPath, fontEnv);
+
+#ifdef NX_TRANS_TEST
+        fprintf(stderr, "_NXGetFontPath: Using NX font path [%s].\n", _NXFontPath);
+#endif
+
+        return _NXFontPath;
+    }
+
+_NXGetFontPathError:
+
+    strcpy(_NXFontPath, path);
+
+#ifdef NX_TRANS_TEST
+    fprintf(stderr, "_NXGetFontPath: Using default font path [%s].\n", _NXFontPath);
+#endif
+
+    return _NXFontPath;
+}
+
 #endif
 
 #ifdef PANORAMIX
@@ -261,10 +331,13 @@ doOpenFont(client, c)
                *newname;
     int         newlen;
     int		aliascount = 20;
+    char nxagentOrigFontName[256];
+    int nxagentOrigFontNameLen;
+
     /*
      * Decide at runtime what FontFormat to use.
      */
-    Mask FontFormat =
+    Mask FontFormat = 
 
 	((screenInfo.imageByteOrder == LSBFirst) ?
 	    BitmapFormatByteOrderLSB : BitmapFormatByteOrderMSB) |
@@ -292,6 +365,13 @@ doOpenFont(client, c)
 
 	BitmapFormatScanlineUnit8;
 
+
+    nxagentOrigFontNameLen = (c -> origFontNameLen < 256) ? c -> origFontNameLen : 255;
+
+    memcpy(nxagentOrigFontName, c -> origFontName, nxagentOrigFontNameLen);
+
+    nxagentOrigFontName[nxagentOrigFontNameLen] = 0;
+
     if (client->clientGone)
     {
 	if (c->current_fpe < c->num_fpes)
@@ -364,11 +444,7 @@ doOpenFont(client, c)
 	    pScr = screenInfo.screens[i];
 	    if (pScr->RealizeFont)
 	    {
-#ifdef NXAGENT_FONTMATCH
-	        char nxagentOrigFontName[256];
-		memcpy(nxagentOrigFontName, c->origFontName, (c->origFontNameLen<256)?c->origFontNameLen:255);
-		nxagentOrigFontName[(c->origFontNameLen<256)?c->origFontNameLen:255] = 0;
-#endif
+
                 /* NXAGENT uses useless screen pointer to pass the original font name
                 *  to realizeFont, could be a source of problems in the future.
                 */
@@ -376,11 +452,7 @@ doOpenFont(client, c)
 		if (!(*pScr->RealizeFont) ((ScreenPtr)nxagentOrigFontName, pfont))
 		{
 		    CloseFont (pfont, (Font) 0);
-#ifdef NXAGENT_FONTMATCH
 		    err=BadFontName;
-#else
-		    err = AllocError;
-#endif
 		    goto bail;
 		}
 	    }
@@ -402,7 +474,7 @@ doOpenFont(client, c)
       }
     }
     if (patternCache && pfont != c->non_cachable_font)
-	CacheFontPattern(patternCache, c->origFontName, c->origFontNameLen,
+	CacheFontPattern(patternCache, nxagentOrigFontName, nxagentOrigFontNameLen,
 			 pfont);
 bail:
     if (err != Successful && c->client != serverClient) {
@@ -622,7 +694,7 @@ QueryFont(pFont, pReply, nProtoCCIStructs)
 	    chars[i++] = r;
 	    chars[i++] = c;
 	}
-	(*pFont->get_metrics) (pFont, ncols, chars,
+	(*pFont->get_metrics) (pFont, ncols, chars, 
 				TwoD16Bit, &count, charInfos);
 	i = 0;
 	for (i = 0; i < (int) count && ninfos < nProtoCCIStructs; i++) {
@@ -744,7 +816,7 @@ doListFontsAndAliases(client, c)
 #ifdef NXAGENT_DEBUG
                         fprintf(stderr, " NXdixfonts: doListFont (3): client [%lx] sleeping.\n", client);
 #endif
-                    }
+		    }
 		    return TRUE;
 		}
 		if (err == FontNameAlias) {
@@ -880,27 +952,24 @@ finish:
 	    reply.nFonts--;
 	else
 	{
-#ifdef NXAGENT_FONTMATCH
 	    {
 	      /* dirty hack: don't list to client fonts not existing on the remote side */
 	      char tmp[256];
 
 	      memcpy(tmp, names->names[i], names->length[i]);
 	      tmp[ names->length[i] ] = 0;
-/* 	      fprintf(stderr, "doListFontsAndAliases, names->names[%d]: %s\n", i, tmp); */
 
 	      if (nxagentFontLookUp(tmp) == 0)
 		{
-# ifdef NXAGENT_FONTMATCH_DEBUG
+#ifdef NXAGENT_FONTMATCH_DEBUG
 		  fprintf(stderr, "doListFontsAndAliases:\n");
 		  fprintf(stderr, "      removing font: %s \n", tmp);
-# endif
+#endif
 		  reply.nFonts--;
 		  stringLens -= names->length[i];
 		  continue;
 		}
 	    }
-#endif
 	    *bufptr++ = names->length[i];
 	    memmove( bufptr, names->names[i], names->length[i]);
 	    bufptr += names->length[i];
@@ -941,7 +1010,7 @@ ListFonts(client, pattern, length, max_names)
     int         i;
     LFclosurePtr c;
 
-    /*
+    /* 
      * The right error to return here would be BadName, however the
      * specification does not allow for a Name error on this request.
      * Perhaps a better solution would be to return a nil list, i.e.
@@ -958,7 +1027,7 @@ ListFonts(client, pattern, length, max_names)
 	xfree(c);
 	return BadAlloc;
     }
-    c->names = MakeFontNamesRecord(max_names < 100 ? max_names : 100);
+    c->names = MakeFontNamesRecord(max_names < nxagentMaxFontNames ? max_names : nxagentMaxFontNames);
     if (!c->names)
     {
 	xfree(c->fpe_list);
@@ -1143,16 +1212,14 @@ doListFontsWithInfo(client, c)
 		namelen = strlen(name);
 	    }
 
-#ifdef NXAGENT_FONTMATCH
 	   if (nxagentFontLookUp(name) == 0)
 	   {
-# ifdef NXAGENT_FONTMATCH_DEBUG
+#ifdef NXAGENT_FONTMATCH_DEBUG
 	      fprintf(stderr, "doListFontsAndAliases (with info):\n");
 	      fprintf(stderr, "      removing font: %s \n", name);
-# endif
+#endif
 	       continue;
            }
-#endif
 
 	    length = sizeof(*reply) + pFontInfo->nprops * sizeof(xFontProp);
 	    reply = c->reply;
@@ -1167,7 +1234,6 @@ doListFontsWithInfo(client, c)
 		c->reply = reply;
 		c->length = length;
 	    }
-
 	    reply->type = X_Reply;
 	    reply->length = (sizeof *reply - sizeof(xGenericReply) +
 			     pFontInfo->nprops * sizeof(xFontProp) +
@@ -1238,7 +1304,7 @@ StartListFontsWithInfo(client, length, pattern, max_names)
     int		    i;
     LFWIclosurePtr  c;
 
-    /*
+    /* 
      * The right error to return here would be BadName, however the
      * specification does not allow for a Name error on this request.
      * Perhaps a better solution would be to return a nil list, i.e.
@@ -1469,11 +1535,11 @@ doPolyText(client, c)
 			err = BadAlloc;
 			goto bail;
 		    }
-#ifdef NXAGENT_MVFB
+
                     pGC->tileIsPixel = TRUE;
                     pGC->tile.pixel = 0;
                     pGC->stipple = NullPixmap;
-#endif
+
 		    if ((err = CopyGC(c->pGC, pGC, GCFunction |
 				      GCPlaneMask | GCForeground |
 				      GCBackground | GCFillStyle |
@@ -1493,7 +1559,7 @@ doPolyText(client, c)
 		    origGC = c->pGC;
 		    c->pGC = pGC;
 		    ValidateGC(c->pDraw, c->pGC);
-
+		    
 		    c->slept = TRUE;
 		    ClientSleep(client,
 		    	     (ClientSleepProcPtr)doPolyText,
@@ -1678,11 +1744,11 @@ doImageText(client, c)
 		err = BadAlloc;
 		goto bail;
 	    }
-#ifdef NXAGENT_MVFB
-                    pGC->tileIsPixel = TRUE;
-                    pGC->tile.pixel = 0;
-                    pGC->stipple = NullPixmap;
-#endif
+
+            pGC->tileIsPixel = TRUE;
+            pGC->tile.pixel = 0;
+            pGC->stipple = NullPixmap;
+
 	    if ((err = CopyGC(c->pGC, pGC, GCFunction | GCPlaneMask |
 			      GCForeground | GCBackground | GCFillStyle |
 			      GCTile | GCStipple | GCTileStipXOrigin |
@@ -1885,11 +1951,11 @@ SetFontPathElements(npaths, paths, bad, persist)
 	if (fpe_functions[i].set_path_hook)
 	    (*fpe_functions[i].set_path_hook) ();
     }
-    for (i = 0; i < npaths; i++)
+    for (i = 0; i < npaths; i++) 
     {
 	len = (unsigned int) (*cp++);
 
-	if (len == 0)
+	if (len == 0) 
 	{
 	    if (persist)
 		ErrorF ("Removing empty element from the valid list of fontpaths\n");
@@ -1903,10 +1969,10 @@ SetFontPathElements(npaths, paths, bad, persist)
 	     * them, though it'd muck up refcounting
 	     */
 	    fpe = find_existing_fpe(font_path_elements, num_fpes, cp, len);
-	    if (fpe)
+	    if (fpe) 
 	    {
 		err = (*fpe_functions[fpe->type].reset_fpe) (fpe);
-		if (err == Successful)
+		if (err == Successful) 
 		{
 		    UseFPE(fpe);/* since it'll be decref'd later when freed
 				 * from the old list */
@@ -1918,20 +1984,20 @@ SetFontPathElements(npaths, paths, bad, persist)
 	    if (!fpe)
 	    {
 		fpe = (FontPathElementPtr) xalloc(sizeof(FontPathElementRec));
-		if (!fpe)
+		if (!fpe) 
 		{
 		    err = BadAlloc;
 		    goto bail;
 		}
 		fpe->name = (char *) xalloc(len + 1);
-		if (!fpe->name)
+		if (!fpe->name) 
 		{
 		    xfree(fpe);
 		    err = BadAlloc;
 		    goto bail;
 		}
 		fpe->refcount = 1;
-
+    
 		strncpy(fpe->name, (char *) cp, (int) len);
 		fpe->name[len] = '\0';
 		fpe->name_length = len;
@@ -2013,11 +2079,19 @@ SetDefaultFontPath(path)
                 bad;
 
     /* get enough for string, plus values -- use up commas */
+#ifdef NX_TRANS_SOCKET
+    len = strlen(_NXGetFontPath(path)) + 1;
+#else
     len = strlen(path) + 1;
+#endif
     nump = cp = newpath = (unsigned char *) ALLOCATE_LOCAL(len);
     if (!newpath)
 	return BadAlloc;
+#ifdef NX_TRANS_SOCKET
+    pp = (unsigned char *) _NXGetFontPath(path);
+#else
     pp = (unsigned char *) path;
+#endif
     cp++;
     while (*pp) {
 	if (*pp == ',') {
@@ -2112,7 +2186,7 @@ InitFonts ()
 	PrinterFontRegisterFpeFunctions();
 	FontFileCheckRegisterFpeFunctions();
 	check_fs_register_fpe_functions();
-    } else
+    } else 
 #endif
     {
 #ifdef KDRIVESERVER
@@ -2148,7 +2222,7 @@ GetClientResolutions (num)
 	pScreen = screenInfo.screens[0];
 	res.x_resolution = (pScreen->width * 25.4) / pScreen->mmWidth;
 	/*
-	 * XXX - we'll want this as long as bitmap instances are prevalent
+	 * XXX - we'll want this as long as bitmap instances are prevalent 
 	 so that we can match them from scalable fonts
 	 */
 	if (res.x_resolution < 88)
@@ -2173,20 +2247,20 @@ GetClientResolutions (num)
  */
 
 int
-RegisterFPEFunctions(NameCheckFunc name_func,
-		     InitFpeFunc init_func,
-		     FreeFpeFunc free_func,
-		     ResetFpeFunc reset_func,
-		     OpenFontFunc open_func,
-		     CloseFontFunc close_func,
-		     ListFontsFunc list_func,
-		     StartLfwiFunc start_lfwi_func,
-		     NextLfwiFunc next_lfwi_func,
-		     WakeupFpeFunc wakeup_func,
-		     ClientDiedFunc client_died,
-		     LoadGlyphsFunc load_glyphs,
-		     StartLaFunc start_list_alias_func,
-		     NextLaFunc next_list_alias_func,
+RegisterFPEFunctions(NameCheckFunc name_func, 
+		     InitFpeFunc init_func, 
+		     FreeFpeFunc free_func, 
+		     ResetFpeFunc reset_func, 
+		     OpenFontFunc open_func, 
+		     CloseFontFunc close_func, 
+		     ListFontsFunc list_func, 
+		     StartLfwiFunc start_lfwi_func, 
+		     NextLfwiFunc next_lfwi_func, 
+		     WakeupFpeFunc wakeup_func, 
+		     ClientDiedFunc client_died, 
+		     LoadGlyphsFunc load_glyphs, 
+		     StartLaFunc start_list_alias_func, 
+		     NextLaFunc next_list_alias_func, 
 		     SetPathFunc set_path_func)
 {
     FPEFunctions *new;
@@ -2365,7 +2439,6 @@ dump_char_ascii(cip)
 #endif
 
 
-#ifdef NXAGENT_FONTMATCH
 typedef struct
 {
    LFclosurePtr c;
@@ -2742,7 +2815,7 @@ nxOpenFont(client, fid, flags, lenfname, pfontname)
     c->num_fpes = num_fpes;
     c->current.patlen = lenfname;
     c->current.current_fpe = 0;
-    c->current.max_names = 10000;
+    c->current.max_names = nxagentMaxFontNames;
     c->current.list_started = FALSE;
     c->current.private = 0;
     c->haveSaved = FALSE;
@@ -2806,4 +2879,5 @@ nxOpenFont(client, fid, flags, lenfname, pfontname)
     return Success;
 }
 
-#endif
+
+#endif /* #ifdef NXAGENT_UPGRADE */
