@@ -1,6 +1,12 @@
+#ifdef NXAGENT_UPGRADE
+
+#include "X/NXpicture.c"
+
+#else
+
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2004 NoMachine, http://www.nomachine.com.          */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
 /* NXAGENT, NX protocol compression and NX extensions to this software    */
 /* are copyright of NoMachine. Redistribution and use of the present      */
@@ -9,13 +15,15 @@
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
 /**************************************************************************/
 
 /*
+ * $XFree86: xc/programs/Xserver/render/picture.c,v 1.30 2003/01/26 16:40:43 eich Exp $
+ *
  * Copyright  2000 SuSE, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -53,9 +61,12 @@
 #include "servermd.h"
 #include "NXpicturestr.h"
 
-#include "Pixmap.h"
+#include "Screen.h"
+#include "Pixmaps.h"
 #include "Drawable.h"
 
+void *nxagentMatchingFormats(PictFormatPtr pForm);
+
 int		PictureScreenPrivateIndex = -1;
 int		PictureWindowPrivateIndex;
 int		PictureGeneration;
@@ -176,7 +187,11 @@ addFormat (FormatInitRec    formats[256],
 PictFormatPtr
 PictureCreateDefaultFormats (ScreenPtr pScreen, int *nformatp)
 {
-    int		    nformats, f;
+#ifdef NXAGENT_SERVER
+    int             nformats, f, n;
+#else
+    int             nformats, f;
+#endif
     PictFormatPtr   pFormats;
     FormatInitRec   formats[1024];
     CARD32	    format;
@@ -326,12 +341,21 @@ PictureCreateDefaultFormats (ScreenPtr pScreen, int *nformatp)
     if (!pFormats)
 	return 0;
     memset (pFormats, '\0', nformats * sizeof (PictFormatRec));
+#ifdef NXAGENT_SERVER
+    for (f = 0, n = 0; n < nformats; n++)
+    {
+        pFormats[f].id = FakeClientID (0);
+        pFormats[f].depth = formats[n].depth;
+        format = formats[n].format;
+        pFormats[f].format = format;
+#else
     for (f = 0; f < nformats; f++)
     {
         pFormats[f].id = FakeClientID (0);
-	pFormats[f].depth = formats[f].depth;
-	format = formats[f].format;
-	pFormats[f].format = format;
+        pFormats[f].depth = formats[f].depth;
+        format = formats[f].format;
+        pFormats[f].format = format;
+#endif
 	switch (PICT_FORMAT_TYPE(format)) {
 	case PICT_TYPE_ARGB:
 	    pFormats[f].type = PictTypeDirect;
@@ -388,8 +412,22 @@ PictureCreateDefaultFormats (ScreenPtr pScreen, int *nformatp)
 	    pFormats[f].index.pVisual = &pScreen->visuals[PICT_FORMAT_VIS(format)];
 	    break;
 	}
+
+#ifdef NXAGENT_SERVER
+        if (nxagentMatchingFormats(&pFormats[f]) != NULL)
+        {
+          f++;
+        }
+        else
+        {
+          memset(&pFormats[f], '\0', sizeof(PictFormatRec));
+        } 
+    }
+    *nformatp = f;
+#else
     }
     *nformatp = nformats;
+#endif
     return pFormats;
 }
 
@@ -663,14 +701,6 @@ PictureInit (ScreenPtr pScreen, PictFormatPtr formats, int nformats)
     pScreen->CloseScreen = PictureCloseScreen;
     pScreen->StoreColors = PictureStoreColors;
 
-    /*
-     * This is originally miModifyPixmapHeader()
-     * from miscrinit.c. It is used to recycle
-     * the scratch pixmap for this screen.
-     */
-
-    pScreen->ModifyPixmapHeader = nxagentModifyPixmapHeader;
-
     if (!PictureSetDefaultFilters (pScreen))
     {
 	PictureResetFilters (pScreen);
@@ -797,23 +827,23 @@ CreatePicture (Picture		pid,
     SetPictureToDefaults (pPicture);
     
     if (vmask)
-        *error = ChangePicture (pPicture, vmask, vlist, 0, client);
+	*error = ChangePicture (pPicture, vmask, vlist, 0, client);
     else
-        *error = Success;
+	*error = Success;
     if (*error == Success)
-        *error = (*ps -> CreatePicture) (pPicture);
+	*error = (*ps->CreatePicture) (pPicture);
     if (*error != Success)
     {
-        FreePicture (pPicture, (XID) 0);
-        pPicture = 0;
+	FreePicture (pPicture, (XID) 0);
+	pPicture = 0;
     }
-
     return pPicture;
 }
 
 #define NEXT_VAL(_type) (vlist ? (_type) *vlist++ : (_type) ulist++->val)
 
 #define NEXT_PTR(_type) ((_type) ulist++->ptr)
+
 int
 ChangePicture (PicturePtr	pPicture,
 	       Mask		vmask,
@@ -955,13 +985,9 @@ ChangePicture (PicturePtr	pPicture,
 			pPixmap->refcnt++;
 		    }
 		}
-               
-                error = (*ps -> ChangePictureClip)(pPicture,
-                                         clipType,
-                                         (pointer)pPixmap,
-                                         0);
-
-        	break;
+		error = (*ps->ChangePictureClip)(pPicture, clipType,
+						 (pointer)pPixmap, 0);
+		break;
 	    }
 	case CPGraphicsExposure:
 	    {
@@ -1038,7 +1064,7 @@ ChangePicture (PicturePtr	pPicture,
 	    break;
 	}
     }
-    (*ps -> ChangePicture)(pPicture, maskQ);
+    (*ps->ChangePicture) (pPicture, maskQ);
     return error;
 }
 
@@ -1054,16 +1080,12 @@ SetPictureClipRects (PicturePtr	pPicture,
     RegionPtr		clientClip;
     int			result;
 
-
     clientClip = RECTS_TO_REGION(pScreen,
 				 nRect, rects, CT_UNSORTED);
     if (!clientClip)
 	return BadAlloc;
-    result = (*ps->ChangePictureClip) (pPicture,
-                                       CT_REGION,
-                                       (pointer) clientClip,
-                                       0);
-    
+    result =(*ps->ChangePictureClip) (pPicture, CT_REGION, 
+				      (pointer) clientClip, 0);
     if (result == Success)
     {
 	pPicture->clipOrigin.x = xOrigin;
@@ -1071,7 +1093,6 @@ SetPictureClipRects (PicturePtr	pPicture,
 	pPicture->stateChanges |= CPClipXOrigin|CPClipYOrigin|CPClipMask;
 	pPicture->serialNumber |= GC_CHANGE_SERIAL_BIT;
     }
-
     return result;
 }
 
@@ -1144,7 +1165,7 @@ FreePicture (pointer	value,
 	if (pPicture->alphaMap)
 	    FreePicture ((pointer) pPicture->alphaMap, (XID) 0);
 	(*ps->DestroyPicture) (pPicture);
-        (*ps->DestroyPictureClip) (pPicture);
+	(*ps->DestroyPictureClip) (pPicture);
 	if (pPicture->transform)
 	    xfree (pPicture->transform);
 	if (pPicture->pDrawable->type == DRAWABLE_WINDOW)
@@ -1214,36 +1235,28 @@ CompositePicture (CARD8		op,
 }
 
 void
-CompositeGlyphs (CARD8          op,
-                 PicturePtr     pSrc,
-                 PicturePtr     pDst,
-                 PictFormatPtr  maskFormat,
-                 INT16          xSrc,
-                 INT16          ySrc,
-                 int            nlist,
-                 GlyphListPtr   lists,
-                 GlyphPtr       *glyphs)
+CompositeGlyphs (CARD8		op,
+		 PicturePtr	pSrc,
+		 PicturePtr	pDst,
+		 PictFormatPtr	maskFormat,
+		 INT16		xSrc,
+		 INT16		ySrc,
+		 int		nlist,
+		 GlyphListPtr	lists,
+		 GlyphPtr	*glyphs)
 {
-  PictureScreenPtr ps = GetPictureScreen(pDst -> pDrawable -> pScreen);
+    PictureScreenPtr	ps = GetPictureScreen(pDst->pDrawable->pScreen);
+    
+    ValidatePicture (pSrc);
+    ValidatePicture (pDst);
 
-  ValidatePicture (pSrc);
-  ValidatePicture (pDst);
+    #ifdef TEST
+    fprintf(stderr, "CompositeGlyphs: Going to composite glyphs with "
+		"source at [%p] and destination at [%p].\n",
+		    (void *) pSrc, (void *) pDst);
+    #endif
 
-  #ifdef TEST
-  fprintf(stderr, "CompositeGlyphs: Going to composite glyphs with "
-              "source at [%p] and destination at [%p].\n",
-                  (void *) pSrc, (void *) pDst);
-  #endif
-
-  (*ps -> Glyphs) (op,
-                   pSrc,
-                   pDst,
-                   maskFormat,
-                   xSrc,
-                   ySrc,
-                   nlist,
-                   lists,
-                   glyphs);
+    (*ps->Glyphs) (op, pSrc, pDst, maskFormat, xSrc, ySrc, nlist, lists, glyphs);
 }
 
 void
@@ -1254,7 +1267,7 @@ CompositeRects (CARD8		op,
 		xRectangle      *rects)
 {
     PictureScreenPtr	ps = GetPictureScreen(pDst->pDrawable->pScreen);
-
+    
     ValidatePicture (pDst);
     (*ps->CompositeRects) (op, pDst, color, nRect, rects);
 }
@@ -1368,8 +1381,6 @@ PictureTransformPoint (PictTransformPtr transform,
     return TRUE;
 }
 
-#ifdef NXAGENT_RECONNECT
-
 #ifndef True
 # define True 1
 #endif
@@ -1378,14 +1389,9 @@ PictureTransformPoint (PictTransformPtr transform,
 # define False 0
 #endif
 
-Bool nxagentRecreatePictFormats(void*);
-void nxagentReConnectPictFormat(void*, XID, void*);
+void nxagentReconnectPictFormat(void*, XID, void*);
 
-extern void nxagentQueryFormats(void);
-
-extern ScreenPtr nxagentDummyScreen;
-
-Bool nxagentReCreatePictFormats(void *p)
+Bool nxagentReconnectAllPictFormat(void *p)
 {
   PictFormatPtr formats_old, formats;
   int nformats, nformats_old;
@@ -1394,18 +1400,18 @@ Bool nxagentReCreatePictFormats(void *p)
   int i, n;
   CARD32 type, a, r, g, b;
 
-#if defined(NXAGENT_RECONNECT_DEBUG) || defined(NXAGENT_RECONNECT_PICTFORMAT_DEBUG)
-  fprintf(stderr, "nxagentReCreatePictFormats\n");
-#endif
+  #ifdef DEBUG
+  fprintf(stderr, "nxagentReconnectAllPictFormat\n");
+  #endif
 
-  formats_old = GetPictureScreen(nxagentDummyScreen) -> formats;
-  nformats_old = GetPictureScreen(nxagentDummyScreen) -> nformats;
+  formats_old = GetPictureScreen(nxagentDefaultScreen) -> formats;
+  nformats_old = GetPictureScreen(nxagentDefaultScreen) -> nformats;
 
   /*
    * TODO: We could copy PictureCreateDefaultFormats,
    *       in order not to waste ID with FakeClientID().
    */
-  formats = PictureCreateDefaultFormats (nxagentDummyScreen, &nformats);
+  formats = PictureCreateDefaultFormats (nxagentDefaultScreen, &nformats);
 
   if (!formats)
     return FALSE;
@@ -1444,7 +1450,6 @@ Bool nxagentReCreatePictFormats(void *p)
     {
       if (formats_old[n].format == formats[i].format &&
           formats_old[n].type == formats[i].type &&
-          formats_old[n].depth == formats[i].depth &&
           formats_old[n].direct.red == formats[i].direct.red &&
           formats_old[n].direct.green == formats[i].direct.green &&
           formats_old[n].direct.blue == formats[i].direct.blue &&
@@ -1453,8 +1458,17 @@ Bool nxagentReCreatePictFormats(void *p)
           formats_old[n].direct.blueMask == formats[i].direct.blueMask &&
           formats_old[n].direct.alpha == formats[i].direct.alpha &&
           formats_old[n].direct.alphaMask == formats[i].direct.alphaMask)
-      {  
-        matched = True;
+      {
+       /*
+        * Regard depth 16 and 15 as were the same, if all other values match.
+        */
+
+        if ((formats_old[n].depth == formats[i].depth) ||
+               ((formats_old[n].depth == 15 || formats_old[n].depth == 16) &&
+                    (formats[i].depth == 15 || formats[i].depth == 16)))
+        {
+          matched = True;
+        }
       }
     }
     if (!matched)
@@ -1464,33 +1478,35 @@ Bool nxagentReCreatePictFormats(void *p)
 
   /* TODO: Perhaps do i have to do PictureFinishInit ?. */
   /* TODO: We have to check for new Render protocol version. */
-  nxagentQueryFormats();
 
-#if 0
   for (i = 0; (i < MAXCLIENTS) && (success); i++)
   {
     if (clients[i])
     {
-      FindClientResourcesByType(clients[i],
-                                PictFormatType,
-                                nxagentReConnectPictFormat,
-                                &success);
+      FindClientResourcesByType(clients[i], PictFormatType, nxagentReconnectPictFormat, &success);
     }
   }
-#endif
+
   return success;
 }
 
-#if 0
-void nxagentReConnectPictFormat(void *p0, XID x1, void *p2)
+/*
+ * It seem we don't have nothing
+ * to do for reconnect PictureFormat.
+ */
+
+void nxagentReconnectPictFormat(void *p0, XID x1, void *p2)
 {
-  PictFormatPtr pFormat = (PictFormatPtr)p0;
-  Bool *pBool = (Bool*)p2;
+  PictFormatPtr pFormat;
+  Bool *pBool;
 
- fprintf(stderr, ""); 
+  pFormat = (PictFormatPtr)p0;
+  pBool = (Bool*)p2;
 
+  #ifdef DEBUG
+  fprintf(stderr, "nxagentReconnectPictFormat.\n");
+  #endif
 }
-#endif
 
-#endif
 
+#endif /* #ifdef NXAGENT_UPGRADE */
