@@ -1,6 +1,12 @@
+#ifdef NXAGENT_UPGRADE
+
+#include "X/NXproperty.c"
+
+#else
+
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2004 NoMachine, http://www.nomachine.com.          */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
 /* NXAGENT, NX protocol compression and NX extensions to this software    */
 /* are copyright of NoMachine. Redistribution and use of the present      */
@@ -9,12 +15,13 @@
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
 /**************************************************************************/
 
+/* $XFree86: xc/programs/Xserver/dix/property.c,v 3.12 2002/02/19 11:09:22 alanh Exp $ */
 /***********************************************************
 
 Copyright 1987, 1998  The Open Group
@@ -44,13 +51,13 @@ Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
+Permission to use, copy, modify, and distribute this software and its 
+documentation for any purpose and without fee is hereby granted, 
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
+both that copyright notice and this permission notice appear in 
 supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.
+software without specific, written prior permission.  
 
 DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
@@ -61,6 +68,7 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ******************************************************************/
+/* $Xorg: property.c,v 1.4 2001/02/09 02:04:40 xorgcvs Exp $ */
 
 #include "X.h"
 #define NEED_REPLIES
@@ -80,6 +88,10 @@ SOFTWARE.
 #include "lbxtags.h"
 #endif
 
+#include "Options.h"
+#include "Rootless.h"
+#include "Client.h"
+
 #if defined(LBX) || defined(LBX_COMPAT)
 int fWriteToClient(client, len, buf)
     ClientPtr   client;
@@ -207,7 +219,7 @@ found:
 	    event.u.property.atom = props[i]->propertyName;	
 	    event.u.property.time = currentTime.milliseconds;
 	    DeliverEvents(pWin, &event, 1, (WindowPtr)NULL);
-
+	
             props[i]->propertyName = atoms[(i + delta) % stuff->nAtoms];
 	}
     }
@@ -215,10 +227,10 @@ found:
     return Success;
 }
 
-int
+int 
 ProcChangeProperty(client)
     ClientPtr client;
-{
+{	      
     WindowPtr pWin;
     char format, mode;
     unsigned long len;
@@ -258,10 +270,8 @@ ProcChangeProperty(client)
 
     if (pWin == NULL)
 #endif
-       pWin = (WindowPtr)SecurityLookupWindow(stuff->window, client,
+    pWin = (WindowPtr)SecurityLookupWindow(stuff->window, client,
 					   SecurityWriteAccess);
-
-
     if (!pWin)
 	return(BadWindow);
     if (!ValidAtom(stuff->property))
@@ -309,7 +319,19 @@ ProcChangeProperty(client)
     if (err != Success)
 	return err;
     else
-	return client->noClientException;
+    {
+      if (nxagentOption(Rootless) == 1)
+      {
+        nxagentExportProperty(pWin, stuff->property, stuff->type, (int) format,
+                                  (int) mode, len, (pointer) &stuff[1]);
+      }
+
+      nxagentGuessClientHint(client, stuff->property, (char *) &stuff[1]);
+
+      nxagentGuessShadowHint(client, stuff->property);
+
+      return client->noClientException;
+    }
 }
 
 int
@@ -571,7 +593,7 @@ ProcGetProperty(client)
     prevProp = (PropertyPtr)NULL;
     while (pProp)
     {
-	if (pProp->propertyName == stuff->property)
+	if (pProp->propertyName == stuff->property) 
 	    break;
 	prevProp = pProp;
 	pProp = pProp->next;
@@ -579,7 +601,7 @@ ProcGetProperty(client)
 
     reply.type = X_Reply;
     reply.sequenceNumber = client->sequence;
-    if (!pProp)
+    if (!pProp) 
 	return NullPropertyReply(client, None, 0, &reply);
 
 #ifdef XCSECURITY
@@ -616,18 +638,18 @@ ProcGetProperty(client)
 	return(Success);
     }
 #ifdef LBX
-    /* make sure we have the current value */
+    /* make sure we have the current value */                       
     if (pProp->tag_id && pProp->owner_pid) {
 	LbxStallPropRequest(client, pProp);
 	return client->noClientException;
-    }
+    }                                              
 #endif
 
 /*
  *  Return type, format, value to client
  */
     n = (pProp->format/8) * pProp->size; /* size (bytes) of prop */
-    ind = stuff->longOffset << 2;
+    ind = stuff->longOffset << 2;        
 
    /* If longOffset is invalid such that it causes "len" to
 	    be negative, it's a value error. */
@@ -895,3 +917,5 @@ ProcDeleteProperty(client)
     else
 	return(result);
 }
+
+#endif /* #ifdef NXAGENT_UPGRADE */
