@@ -1,21 +1,12 @@
-/**************************************************************************/
-/*                                                                        */
-/* Copyright (c) 2001, 2004 NoMachine, http://www.nomachine.com.          */
-/*                                                                        */
-/* NXAGENT, NX protocol compression and NX extensions to this software    */
-/* are copyright of NoMachine. Redistribution and use of the present      */
-/* software is allowed according to terms specified in the file LICENSE   */
-/* which comes in the source distribution.                                */
-/*                                                                        */
-/* Check http://www.nomachine.com/licensing.html for applicability.       */
-/*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
-/*                                                                        */
-/* All rights reserved.                                                   */
-/*                                                                        */
-/**************************************************************************/
+#ifdef NXAGENT_UPGRADE
+
+#include "X/NXrender.c"
+
+#else
 
 /*
+ * $XFree86: xc/programs/Xserver/render/render.c,v 1.26 2003/02/14 18:15:21 dawes Exp $
+ *
  * Copyright  2000 SuSE, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -38,6 +29,23 @@
  * Author:  Keith Packard, SuSE, Inc.
  */
 
+/**************************************************************************/
+/*                                                                        */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
+/*                                                                        */
+/* NXAGENT, NX protocol compression and NX extensions to this software    */
+/* are copyright of NoMachine. Redistribution and use of the present      */
+/* software is allowed according to terms specified in the file LICENSE   */
+/* which comes in the source distribution.                                */
+/*                                                                        */
+/* Check http://www.nomachine.com/licensing.html for applicability.       */
+/*                                                                        */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
+/*                                                                        */
+/* All rights reserved.                                                   */
+/*                                                                        */
+/**************************************************************************/
+
 #define NEED_REPLIES
 #define NEED_EVENTS
 #include "X.h"
@@ -66,7 +74,9 @@
 #include "Trap.h"
 
 #include "Render.h"
-#include "Pixmap.h"
+#include "Pixmaps.h"
+#include "Options.h"
+#include "Screen.h"
 
 /*
  * Set here the required log level.
@@ -77,34 +87,43 @@
 #undef  TEST
 #undef  DEBUG
 
+/*
+ * From NXmiglyph.c.
+ */
+
+void miGlyphExtents(int nlist, GlyphListPtr list,
+                        GlyphPtr *glyphs, BoxPtr extents);
+
 /*
  * Functions from Render.c.
  */
 
-extern int  nxagentCreatePicture(PicturePtr, Mask);
-extern void nxagentDestroyPicture(PicturePtr pPicture);
-extern void nxagentChangePicture(PicturePtr, Mask);
-extern int  nxagentChangePictureClip(PicturePtr, int, int, xRectangle *, int, int);
-extern void nxagentComposite(CARD8, PicturePtr, PicturePtr, PicturePtr, INT16, INT16,
-                                 INT16, INT16, INT16, INT16, CARD16, CARD16);
-extern void nxagentCompositeRects(CARD8, PicturePtr, xRenderColor *, int, xRectangle *);
-extern void nxagentCreateGlyphSet(GlyphSetPtr glyphSet);
-extern void nxagentReferenceGlyphSet(GlyphSetPtr glyphSet);
-extern void nxagentAddGlyphs(GlyphSetPtr glyphSet, Glyph *gids, xGlyphInfo *gi,
-                                 int nglyphs,  CARD8 *images, int sizeImages);
-
-extern void nxagentFreeGlyphs(GlyphSetPtr glyphSet, CARD32 *gids, int nglyph);
-extern void nxagentFreeGlyphSet(GlyphSetPtr glyphSet);
-extern void nxagentSetPictureTransform(PicturePtr pPicture, pointer transform);
-extern void nxagentSetPictureFilter(PicturePtr pPicture, char *filter, int name_size,
-                                        pointer params, int nparams);
+int  nxagentCursorSaveRenderInfo(ScreenPtr, CursorPtr);
+void nxagentCursorPostSaveRenderInfo(CursorPtr, ScreenPtr, PicturePtr, int, int);
+int  nxagentRenderRealizeCursor(ScreenPtr, CursorPtr);
+int  nxagentCreatePicture(PicturePtr, Mask);
+void nxagentDestroyPicture(PicturePtr pPicture);
+void nxagentChangePicture(PicturePtr, Mask);
+int  nxagentChangePictureClip(PicturePtr, int, int, xRectangle *, int, int);
+void nxagentComposite(CARD8, PicturePtr, PicturePtr, PicturePtr, INT16, INT16,
+                          INT16, INT16, INT16, INT16, CARD16, CARD16);
+void nxagentCompositeRects(CARD8, PicturePtr, xRenderColor *, int, xRectangle *);
+void nxagentCreateGlyphSet(GlyphSetPtr glyphSet);
+void nxagentReferenceGlyphSet(GlyphSetPtr glyphSet);
+void nxagentFreeGlyphs(GlyphSetPtr glyphSet, CARD32 *gids, int nglyph);
+void nxagentFreeGlyphSet(GlyphSetPtr glyphSet);
+void nxagentSetPictureTransform(PicturePtr pPicture, pointer transform);
+void nxagentSetPictureFilter(PicturePtr pPicture, char *filter, int name_size,
+                                 pointer params, int nparams);
+void nxagentTrapezoids(CARD8 op, PicturePtr pSrc, PicturePtr pDst, PictFormatPtr maskFormat,
+                           INT16 xSrc, INT16 ySrc, int ntrap, xTrapezoid *traps);
 
 /*
- * The void pointer is actually XGlyphElt8.
+ * The void pointer is actually a XGlyphElt8.
  */
 
-extern void nxagentGlyphs(CARD8, PicturePtr, PicturePtr, PictFormatPtr,
-                              INT16, INT16, int, void *, int);
+void nxagentGlyphs(CARD8, PicturePtr, PicturePtr, PictFormatPtr,
+                       INT16, INT16, int, void *, int, GlyphPtr *);
 
 static int ProcRenderQueryVersion (ClientPtr pClient);
 static int ProcRenderQueryPictFormats (ClientPtr pClient);
@@ -279,7 +298,7 @@ RenderExtensionInit (void)
     if (!PictureFinishInit ())
 	return;
     RenderClientPrivateIndex = AllocateClientPrivateIndex ();
-    if (!AllocateClientPrivate (RenderClientPrivateIndex,
+    if (!AllocateClientPrivate (RenderClientPrivateIndex, 
 				sizeof (RenderClientRec)))
 	return;
     if (!AddCallback (&ClientStateCallback, RenderClientCallback, 0))
@@ -309,13 +328,13 @@ ProcRenderQueryVersion (ClientPtr client)
 
     pRenderClient->major_version = stuff->majorVersion;
     pRenderClient->minor_version = stuff->minorVersion;
-    
+
     REQUEST_SIZE_MATCH(xRenderQueryVersionReq);
     rep.type = X_Reply;
     rep.length = 0;
     rep.sequenceNumber = client->sequence;
-    rep.majorVersion = RENDER_MAJOR;
-    rep.minorVersion = RENDER_MINOR;
+    rep.majorVersion = nxagentRenderVersionMajor;
+    rep.minorVersion = nxagentRenderVersionMinor;
     if (client->swapped) {
     	swaps(&rep.sequenceNumber, n);
     	swapl(&rep.length, n);
@@ -396,7 +415,7 @@ ProcRenderQueryPictFormats (ClientPtr client)
 #ifdef PANORAMIX
     if (noPanoramiXExtension)
 	numScreens = screenInfo.numScreens;
-    else
+    else 
         numScreens = ((xConnSetup *)ConnectionInfo)->numRoots;
 #else
     numScreens = screenInfo.numScreens;
@@ -425,7 +444,7 @@ ProcRenderQueryPictFormats (ClientPtr client)
 	numSubpixel = 0;
     else
 	numSubpixel = numScreens;
-
+    
     rlength = (sizeof (xRenderQueryPictFormatsReply) +
 	       nformat * sizeof (xPictFormInfo) +
 	       numScreens * sizeof (xPictScreen) +
@@ -443,16 +462,16 @@ ProcRenderQueryPictFormats (ClientPtr client)
     reply->numDepths = ndepth;
     reply->numVisuals = nvisual;
     reply->numSubpixel = numSubpixel;
-
+    
     pictForm = (xPictFormInfo *) (reply + 1);
-
+    
     for (s = 0; s < numScreens; s++)
     {
 	pScreen = screenInfo.screens[s];
 	ps = GetPictureScreenIfSet(pScreen);
 	if (ps)
 	{
-	    for (nformat = 0, pFormat = ps->formats;
+	    for (nformat = 0, pFormat = ps->formats; 
 		 nformat < ps->nformats;
 		 nformat++, pFormat++)
 	    {
@@ -465,12 +484,9 @@ ProcRenderQueryPictFormats (ClientPtr client)
 		pictForm->direct.greenMask = pFormat->direct.greenMask;
 		pictForm->direct.blue = pFormat->direct.blue;
 		pictForm->direct.blueMask = pFormat->direct.blueMask;
-
 		pictForm->direct.alpha = nxagentAlphaEnabled ? pFormat->direct.alpha : 0;
-
 		pictForm->direct.alphaMask = pFormat->direct.alphaMask;
-
-                if (pFormat->type == PictTypeIndexed && pFormat->index.pColormap)
+		if (pFormat->type == PictTypeIndexed && pFormat->index.pColormap)
 		    pictForm->colormap = pFormat->index.pColormap->mid;
 		else
 		    pictForm->colormap = None;
@@ -491,7 +507,7 @@ ProcRenderQueryPictFormats (ClientPtr client)
 	    }
 	}
     }
-
+    
     pictScreen = (xPictScreen *) pictForm;
     for (s = 0; s < numScreens; s++)
     {
@@ -507,8 +523,8 @@ ProcRenderQueryPictFormats (ClientPtr client)
 	    for (v = 0; v < pDepth->numVids; v++)
 	    {
 		pVisual = findVisual (pScreen, pDepth->vids[v]);
-		if (pVisual && (pFormat = PictureMatchVisual (pScreen,
-							      pDepth->depth,
+		if (pVisual && (pFormat = PictureMatchVisual (pScreen, 
+							      pDepth->depth, 
 							      pVisual)))
 		{
 		    pictVisual->visual = pVisual->vid;
@@ -545,7 +561,7 @@ ProcRenderQueryPictFormats (ClientPtr client)
 	pictScreen = (xPictScreen *) pictDepth;
     }
     pictSubpixel = (CARD32 *) pictScreen;
-
+    
     for (s = 0; s < numSubpixel; s++)
     {
 	pScreen = screenInfo.screens[s];
@@ -560,7 +576,7 @@ ProcRenderQueryPictFormats (ClientPtr client)
 	}
 	++pictSubpixel;
     }
-
+    
     if (client->swapped)
     {
 	swaps (&reply->sequenceNumber, n);
@@ -588,7 +604,8 @@ ProcRenderQueryPictIndexValues (ClientPtr client)
     xIndexValue	    *values;
 
     REQUEST_AT_LEAST_SIZE(xRenderQueryPictIndexValuesReq);
-    pFormat = (PictFormatPtr) SecurityLookupIDByType (client,
+
+    pFormat = (PictFormatPtr) SecurityLookupIDByType (client, 
 						      stuff->format,
 						      PictFormatType,
 						      SecurityReadAccess);
@@ -660,7 +677,7 @@ ProcRenderCreatePicture (ClientPtr client)
     LEGAL_NEW_RESOURCE(stuff->pid, client);
     SECURITY_VERIFY_DRAWABLE(pDrawable, stuff->drawable, client,
 			     SecurityWriteAccess);
-    pFormat = (PictFormatPtr) SecurityLookupIDByType (client,
+    pFormat = (PictFormatPtr) SecurityLookupIDByType (client, 
 						      stuff->format,
 						      PictFormatType,
 						      SecurityReadAccess);
@@ -674,7 +691,7 @@ ProcRenderCreatePicture (ClientPtr client)
     len = client->req_len - (sizeof(xRenderCreatePictureReq) >> 2);
     if (Ones(stuff->mask) != len)
 	return BadLength;
- 
+    
     pPicture = CreatePicture (stuff->pid,
 			      pDrawable,
 			      pFormat,
@@ -682,10 +699,8 @@ ProcRenderCreatePicture (ClientPtr client)
 			      (XID *) (stuff + 1),
 			      client,
 			      &error);
-
     if (!pPicture)
-        return error;
-
+	return error;
     nxagentCreatePicture(pPicture, stuff -> mask);
 
     if (!AddResource (stuff->pid, PictureType, (pointer)pPicture))
@@ -707,9 +722,9 @@ ProcRenderChangePicture (ClientPtr client)
     len = client->req_len - (sizeof(xRenderChangePictureReq) >> 2);
     if (Ones(stuff->mask) != len)
 	return BadLength;
-
+    
     error = ChangePicture (pPicture, stuff->mask, (XID *) (stuff + 1),
-                          (DevUnion *) 0, client);
+			  (DevUnion *) 0, client);
     
     nxagentChangePicture(pPicture, stuff->mask);
 
@@ -727,24 +742,19 @@ ProcRenderSetPictureClipRectangles (ClientPtr client)
     REQUEST_AT_LEAST_SIZE(xRenderSetPictureClipRectanglesReq);
     VERIFY_PICTURE (pPicture, stuff->picture, client, SecurityWriteAccess,
 		    RenderErrBase + BadPicture);
-    /* COMMENT:
-     * original KP source uses "sizeof(xRenderChangePictureReq)"
-     * and nothing happened as both structures have the same size.
-     * Probably is an inofensive error by the moment.
-     * Anyway we corrected it.
+    /*
+     * The original code used sizeof(xRenderChangePictureReq).
+     * This was harmless, as both structures have the same size.
+     *
+     * nr = (client->req_len << 2) - sizeof(xRenderChangePictureReq);
      */
     nr = (client->req_len << 2) - sizeof(xRenderSetPictureClipRectanglesReq);
     if (nr & 4)
 	return BadLength;
     nr >>= 3;
-
-    result = SetPictureClipRects (pPicture,
-                                  stuff -> xOrigin,
-                                  stuff -> yOrigin,
-                                  nr,
-                                  (xRectangle *) &stuff[1]);
-
-
+    result = SetPictureClipRects (pPicture, 
+				  stuff->xOrigin, stuff->yOrigin,
+				  nr, (xRectangle *) &stuff[1]);
     nxagentChangePictureClip (pPicture,
                               CT_NONE,
                               nr,
@@ -763,6 +773,7 @@ ProcRenderFreePicture (ClientPtr client)
 {
     PicturePtr	pPicture;
     REQUEST(xRenderFreePictureReq);
+
     REQUEST_SIZE_MATCH(xRenderFreePictureReq);
 
     VERIFY_PICTURE (pPicture, stuff->picture, client, SecurityDestroyAccess,
@@ -796,9 +807,7 @@ PictOpValid (CARD8 op)
 
 #ifdef NXAGENT_PICTURE_ALWAYS_POINTS_TO_VIRTUAL
 
-#define nxagentCompositePredicate(pSrc, pDst) \
-  (((pSrc) -> pDrawable -> type == DRAWABLE_PIXMAP && \
-       (pDst) -> pDrawable -> type == DRAWABLE_PIXMAP) ? TRUE : FALSE)
+#define nxagentCompositePredicate(pSrc, pDst)  TRUE
 
 #else
 
@@ -819,26 +828,34 @@ int nxagentCompositePredicate(PicturePtr pSrc, PicturePtr pDst)
   pPixmap2 = (pDst -> pDrawable -> type == DRAWABLE_PIXMAP ?
                  ((PixmapPtr) pDst -> pDrawable) : NULL);
 
-  if (!pPixmap1 || !pPixmap2)
+  if (pPixmap1 == NULL || pPixmap2 == NULL)
   {
+    #ifdef TEST
     fprintf(stderr, "nxagentCompositePredicate: Case 0.\n");
+    #endif
 
     return FALSE;
   }
   else
   {
+    #ifdef TEST
     fprintf(stderr, "nxagentCompositePredicate: Case 1.\n");
+    #endif
 
-    if (nxagentPixmapIsVirtual(pPixmap1) &&
-            nxagentPixmapIsVirtual(pPixmap2))
+    if (nxagentPixmapIsVirtual(pPixmap1) == 1 &&
+            nxagentPixmapIsVirtual(pPixmap2) == 1)
     {
+      #ifdef TEST
       fprintf(stderr, "nxagentCompositePredicate: Case 2.\n");
+      #endif
 
       return TRUE;
     }
   }
 
+  #ifdef TEST
   fprintf(stderr, "nxagentCompositePredicate: Case 3.\n");
+  #endif
 
   return FALSE;
 }
@@ -857,11 +874,11 @@ ProcRenderComposite (ClientPtr client)
 	client->errorValue = stuff->op;
 	return BadValue;
     }
-    VERIFY_PICTURE (pSrc, stuff->src, client, SecurityReadAccess,
+    VERIFY_PICTURE (pSrc, stuff->src, client, SecurityReadAccess, 
 		    RenderErrBase + BadPicture);
-    VERIFY_ALPHA (pMask, stuff->mask, client, SecurityReadAccess,
+    VERIFY_ALPHA (pMask, stuff->mask, client, SecurityReadAccess, 
 		  RenderErrBase + BadPicture);
-    VERIFY_PICTURE (pDst, stuff->dst, client, SecurityWriteAccess,
+    VERIFY_PICTURE (pDst, stuff->dst, client, SecurityWriteAccess, 
 		    RenderErrBase + BadPicture);
     if (pSrc->pDrawable->pScreen != pDst->pDrawable->pScreen ||
 	(pMask && pSrc->pDrawable->pScreen != pMask->pDrawable->pScreen))
@@ -882,18 +899,18 @@ ProcRenderComposite (ClientPtr client)
                       (void *) pSrc, (void *) pMask, (void *) pDst);
       #endif
 
-      CompositePicture (stuff->op,
-                        pSrc,
-                        pMask,
-                        pDst,
-                        stuff->xSrc,
-                        stuff->ySrc,
-                        stuff->xMask,
-                        stuff->yMask,
-                        stuff->xDst,
-                        stuff->yDst,
-                        stuff->width,
-                        stuff->height);
+    CompositePicture (stuff->op,
+		      pSrc,
+		      pMask,
+		      pDst,
+		      stuff->xSrc,
+		      stuff->ySrc,
+		      stuff->xMask,
+		      stuff->yMask,
+		      stuff->xDst,
+		      stuff->yDst,
+		      stuff->width,
+		      stuff->height);
     }
 
     #else
@@ -982,6 +999,7 @@ ProcRenderTrapezoids (ClientPtr client)
     PicturePtr	pSrc, pDst;
     PictFormatPtr   pFormat;
     REQUEST(xRenderTrapezoidsReq);
+
     REQUEST_AT_LEAST_SIZE(xRenderTrapezoidsReq);
     if (!PictOpValid (stuff->op))
     {
@@ -1013,9 +1031,19 @@ ProcRenderTrapezoids (ClientPtr client)
 	return BadLength;
     ntraps /= sizeof (xTrapezoid);
     if (ntraps)
+    {
+        if (nxagentCompositePredicate(pSrc, pDst))
+        {
 	CompositeTrapezoids (stuff->op, pSrc, pDst, pFormat,
 			     stuff->xSrc, stuff->ySrc,
 			     ntraps, (xTrapezoid *) &stuff[1]);
+        }
+
+        nxagentTrapezoids (stuff->op, pSrc, pDst, pFormat,
+                             stuff->xSrc, stuff->ySrc,
+                             ntraps, (xTrapezoid *) &stuff[1]);
+    }
+
     return client->noClientException;
 }
 
@@ -1026,13 +1054,14 @@ ProcRenderTriangles (ClientPtr client)
     PicturePtr	pSrc, pDst;
     PictFormatPtr   pFormat;
     REQUEST(xRenderTrianglesReq);
+
     REQUEST_AT_LEAST_SIZE(xRenderTrianglesReq);
     if (!PictOpValid (stuff->op))
     {
 	client->errorValue = stuff->op;
 	return BadValue;
     }
-    VERIFY_PICTURE (pSrc, stuff->src, client, SecurityReadAccess,
+    VERIFY_PICTURE (pSrc, stuff->src, client, SecurityReadAccess, 
 		    RenderErrBase + BadPicture);
     VERIFY_PICTURE (pDst, stuff->dst, client, SecurityWriteAccess, 
 		    RenderErrBase + BadPicture);
@@ -1070,15 +1099,16 @@ ProcRenderTriStrip (ClientPtr client)
     PicturePtr	pSrc, pDst;
     PictFormatPtr   pFormat;
     REQUEST(xRenderTrianglesReq);
+
     REQUEST_AT_LEAST_SIZE(xRenderTrianglesReq);
     if (!PictOpValid (stuff->op))
     {
 	client->errorValue = stuff->op;
 	return BadValue;
     }
-    VERIFY_PICTURE (pSrc, stuff->src, client, SecurityReadAccess,
+    VERIFY_PICTURE (pSrc, stuff->src, client, SecurityReadAccess, 
 		    RenderErrBase + BadPicture);
-    VERIFY_PICTURE (pDst, stuff->dst, client, SecurityWriteAccess,
+    VERIFY_PICTURE (pDst, stuff->dst, client, SecurityWriteAccess, 
 		    RenderErrBase + BadPicture);
     if (pSrc->pDrawable->pScreen != pDst->pDrawable->pScreen)
 	return BadMatch;
@@ -1121,7 +1151,7 @@ ProcRenderTriFan (ClientPtr client)
 	client->errorValue = stuff->op;
 	return BadValue;
     }
-    VERIFY_PICTURE (pSrc, stuff->src, client, SecurityReadAccess,
+    VERIFY_PICTURE (pSrc, stuff->src, client, SecurityReadAccess, 
 		    RenderErrBase + BadPicture);
     VERIFY_PICTURE (pDst, stuff->dst, client, SecurityWriteAccess, 
 		    RenderErrBase + BadPicture);
@@ -1177,6 +1207,7 @@ ProcRenderCreateGlyphSet (ClientPtr client)
     PictFormatPtr   format;
     int		    f;
     REQUEST(xRenderCreateGlyphSetReq);
+
     REQUEST_SIZE_MATCH(xRenderCreateGlyphSetReq);
 
     LEGAL_NEW_RESOURCE(stuff->gsid, client);
@@ -1213,7 +1244,6 @@ ProcRenderCreateGlyphSet (ClientPtr client)
     glyphSet = AllocateGlyphSet (f, format);
     if (!glyphSet)
 	return BadAlloc;
-
     if (!AddResource (stuff->gsid, GlyphSetType, (pointer)glyphSet))
 	return BadAlloc;
 
@@ -1297,11 +1327,6 @@ ProcRenderAddGlyphs (ClientPtr client)
     int		    err = BadAlloc;
 
     int             totSizeImages;
-    CARD8           *bitsTemp=NULL;
-    CARD32          *gidsTemp;
-    int             nglyphsTemp;
-    GlyphNewPtr     glyphsTemp;
-    xGlyphInfo      *giTemp=NULL;
 
     REQUEST_AT_LEAST_SIZE(xRenderAddGlyphsReq);
     glyphSet = (GlyphSetPtr) SecurityLookupIDByType (client,
@@ -1334,12 +1359,6 @@ ProcRenderAddGlyphs (ClientPtr client)
     bits = (CARD8 *) (gi + nglyphs);
     remain -= (sizeof (CARD32) + sizeof (xGlyphInfo)) * nglyphs;
 
-    gidsTemp = gids;
-    nglyphsTemp = nglyphs;
-    glyphsTemp = glyphs;
-    bitsTemp = bits;
-    giTemp = gi;
-
     while (remain >= 0 && nglyphs)
     {
 	glyph = AllocateGlyph (gi, glyphSet->fdepth);
@@ -1348,20 +1367,18 @@ ProcRenderAddGlyphs (ClientPtr client)
 	    err = BadAlloc;
 	    goto bail;
 	}
-
+	
 	glyphs->glyph = glyph;
-	glyphs->id = *gids;
-
+	glyphs->id = *gids;	
+	
 	size = glyph->size - sizeof (xGlyphInfo);
 	if (remain < size)
 	    break;
-
-        memcpy ((CARD8 *) (glyph + 1), bits, size);
-
+	memcpy ((CARD8 *) (glyph + 1), bits, size);
+	
 	if (size & 3)
 	    size += 4 - (size & 3);
-
-        bits += size;
+	bits += size;
 	totSizeImages += size;
 	remain -= size;
 	gi++;
@@ -1370,11 +1387,6 @@ ProcRenderAddGlyphs (ClientPtr client)
 	nglyphs--;
     }
 
-    if (giTemp && bitsTemp)
-    {
-      nxagentAddGlyphs(glyphSet, gidsTemp, giTemp, nglyphsTemp, bitsTemp, totSizeImages);
-    }
-
     if (nglyphs || remain)
     {
 	err = BadLength;
@@ -1473,6 +1485,7 @@ ProcRenderCompositeGlyphs (ClientPtr client)
     int		    space;
     int		    size;
     int		    n;
+    
     XGlyphElt8      *elements, *elementsBase;
 
     REQUEST(xRenderCompositeGlyphsReq);
@@ -1484,7 +1497,7 @@ ProcRenderCompositeGlyphs (ClientPtr client)
     case X_RenderCompositeGlyphs16: size = 2; break;
     case X_RenderCompositeGlyphs32: size = 4; break;
     }
-
+	    
     if (!PictOpValid (stuff->op))
     {
 	client->errorValue = stuff->op;
@@ -1525,12 +1538,11 @@ ProcRenderCompositeGlyphs (ClientPtr client)
     end = (CARD8 *) stuff + (client->req_len << 2);
     nglyph = 0;
     nlist = 0;
-
     while (buffer + sizeof (xGlyphElt) < end)
     {
 	elt = (xGlyphElt *) buffer;
 	buffer += sizeof (xGlyphElt);
-
+	
 	if (elt->len == 0xff)
 	{
 	    buffer += 4;
@@ -1574,9 +1586,14 @@ ProcRenderCompositeGlyphs (ClientPtr client)
     {
 	elt = (xGlyphElt *) buffer;
 	buffer += sizeof (xGlyphElt);
-
+	
 	if (elt->len == 0xff)
 	{
+            #ifdef DEBUG
+            fprintf(stderr, "ProcRenderCompositeGlyphs: Glyphset change with base size [%d].\n",
+                        size);
+            #endif
+
 	    if (buffer + sizeof (GlyphSet) < end)
 	    {
 		gs = *(GlyphSet *) buffer;
@@ -1629,7 +1646,6 @@ ProcRenderCompositeGlyphs (ClientPtr client)
 		}
 		buffer += size;
 	    }
-
 	    space = size * elt->len;
 	    if (space & 3)
 		buffer += 4 - (space & 3);
@@ -1640,7 +1656,23 @@ ProcRenderCompositeGlyphs (ClientPtr client)
     if (buffer > end)
 	return BadLength;
 
-    nxagentGlyphs(stuff->op,
+    /*
+     * We need to know the glyphs extents to synchronize
+     * the drawables involved in the composite text ope-
+     * ration. Also we need to synchronize only the back-
+     * ground of the text we are going to render, so the
+     * operations on the framebuffer must be executed
+     * after the X requests.
+     */
+
+    if (pFormat != NULL)
+    {
+      nxagentGlyphsExtents = (BoxPtr) xalloc(sizeof(BoxRec));
+
+      miGlyphExtents(nlist, listsBase, glyphsBase, nxagentGlyphsExtents);
+    }
+
+    nxagentGlyphs(stuff -> op,
                   pSrc,
                   pDst,
                   pFormat,
@@ -1648,9 +1680,10 @@ ProcRenderCompositeGlyphs (ClientPtr client)
                   stuff -> ySrc,
                   nlist,
                   elementsBase,
-                  size);
+                  size,
+                  glyphsBase);
 
-    if (nxagentCompositePredicate(pSrc, pDst))
+    if (nxagentCompositePredicate(pSrc, pDst) == 1)
     {
       #ifdef TEST
       fprintf(stderr, "ProcRenderCompositeGlyphs: Going to composite glyphs with "
@@ -1658,22 +1691,29 @@ ProcRenderCompositeGlyphs (ClientPtr client)
                       (void *) pSrc, (void *) pDst);
       #endif
 
-      CompositeGlyphs (stuff -> op,
-                       pSrc,
-                       pDst,
-                       pFormat,
-                       stuff -> xSrc,
-                       stuff -> ySrc,
-                       nlist,
-                       listsBase,
-                       glyphsBase);
+      CompositeGlyphs(stuff -> op,
+                      pSrc,
+                      pDst,
+                      pFormat,
+                      stuff -> xSrc,
+                      stuff -> ySrc,
+                      nlist,
+                      listsBase,
+                      glyphsBase);
+    }
+
+    if (nxagentGlyphsExtents != NullBox)
+    {
+      xfree(nxagentGlyphsExtents);
+
+      nxagentGlyphsExtents = NullBox;
     }
 
     if (glyphsBase != glyphsLocal)
 	DEALLOCATE_LOCAL (glyphsBase);
     if (listsBase != listsLocal)
 	DEALLOCATE_LOCAL (listsBase);
-
+    
     xfree(elementsBase);
 
     return client->noClientException;
@@ -1685,7 +1725,7 @@ ProcRenderFillRectangles (ClientPtr client)
     PicturePtr	    pDst;
     int             things;
     REQUEST(xRenderFillRectanglesReq);
-
+    
     REQUEST_AT_LEAST_SIZE (xRenderFillRectanglesReq);
     if (!PictOpValid (stuff->op))
     {
@@ -1694,7 +1734,7 @@ ProcRenderFillRectangles (ClientPtr client)
     }
     VERIFY_PICTURE (pDst, stuff->dst, client, SecurityWriteAccess, 
 		    RenderErrBase + BadPicture);
-
+    
     things = (client->req_len << 2) - sizeof(xRenderFillRectanglesReq);
     if (things & 4)
 	return(BadLength);
@@ -1760,14 +1800,17 @@ ProcRenderCreateCursor (ClientPtr client)
     CARD32	    twocolor[3];
     int		    ncolor;
 
+    RealizeCursorProcPtr saveRealizeCursor;
+
     REQUEST_SIZE_MATCH (xRenderCreateCursorReq);
     LEGAL_NEW_RESOURCE(stuff->cid, client);
-    VERIFY_PICTURE (pSrc, stuff->src, client, SecurityReadAccess,
+    
+    VERIFY_PICTURE (pSrc, stuff->src, client, SecurityReadAccess, 
 		    RenderErrBase + BadPicture);
     pScreen = pSrc->pDrawable->pScreen;
     width = pSrc->pDrawable->width;
     height = pSrc->pDrawable->height;
-    if ( stuff->x > width
+    if ( stuff->x > width 
       || stuff->y > height )
 	return (BadMatch);
     argbbits = xalloc (width * height * sizeof (CARD32));
@@ -1823,7 +1866,7 @@ ProcRenderCreateCursor (ClientPtr client)
 	}
 	pPicture = CreatePicture (0, &pPixmap->drawable, pFormat, 0, 0, 
 				  client, &error);
-	if (!pPicture);
+	if (!pPicture)
 	{
 	    xfree (argbbits);
 	    xfree (srcbits);
@@ -1922,6 +1965,20 @@ ProcRenderCreateCursor (ClientPtr client)
     cm.height = height;
     cm.xhot = stuff->x;
     cm.yhot = stuff->y;
+
+    /*
+     * This cursor uses RENDER, so we make sure
+     * that it is allocated in a way that allows
+     * the mi and dix layers to handle it but we
+     * later create it on the server by mirror-
+     * ing the RENDER operation we got from the
+     * client.
+     */
+
+    saveRealizeCursor = pScreen -> RealizeCursor;
+
+    pScreen -> RealizeCursor = nxagentCursorSaveRenderInfo;
+
     pCursor = AllocCursorARGB (srcbits, mskbits, argbbits, &cm,
 			       GetColor(twocolor[0], 16),
 			       GetColor(twocolor[0], 8),
@@ -1929,7 +1986,27 @@ ProcRenderCreateCursor (ClientPtr client)
 			       GetColor(twocolor[1], 16),
 			       GetColor(twocolor[1], 8),
 			       GetColor(twocolor[1], 0));
-    if (pCursor && AddResource(stuff->cid, RT_CURSOR, (pointer)pCursor))
+
+    pScreen -> RealizeCursor = saveRealizeCursor;
+
+    /*
+     * Store into the private data members the
+     * information needed to recreate it at
+     * reconnection. This is done in two steps
+     * as in the first step we don't have the
+     * picture info.
+     */
+
+    if (pCursor == NULL)
+    {
+      return BadAlloc;
+    }
+
+    nxagentCursorPostSaveRenderInfo(pCursor, pScreen, pSrc, stuff -> x, stuff -> y);
+
+    nxagentRenderRealizeCursor(pScreen, pCursor);
+
+    if (AddResource(stuff->cid, RT_CURSOR, (pointer)pCursor))
 	return (client->noClientException);
     return BadAlloc;
 }
@@ -1964,7 +2041,7 @@ ProcRenderQueryFilters (ClientPtr client)
     int				nnames;
     ScreenPtr			pScreen;
     PictureScreenPtr		ps;
-    int	                	i, j;
+    int				i, j;
     int				len;
     int				total_bytes;
     INT16			*aliases;
@@ -1972,7 +2049,7 @@ ProcRenderQueryFilters (ClientPtr client)
 
     REQUEST_SIZE_MATCH(xRenderQueryFiltersReq);
     SECURITY_VERIFY_DRAWABLE(pDrawable, stuff->drawable, client, SecurityReadAccess);
-
+    
     pScreen = pDrawable->pScreen;
     nbytesName = 0;
     nnames = 0;
@@ -2033,7 +2110,7 @@ ProcRenderQueryFilters (ClientPtr client)
 	    strncpy (names, ps->filters[i].name, j);
 	    names += j;
 	}
-
+	
 	/* fill in filter alias names */
 	for (i = 0; i < ps->nfilterAliases; i++)
 	{
@@ -2126,6 +2203,11 @@ ProcRenderCreateAnimCursor (ClientPtr client)
     if (ret != Success)
 	return ret;
     
+    for (i = 0; i < MAXSCREENS; i++)
+    {
+      pCursor -> devPriv[i] = NULL;
+    }
+
     if (AddResource (stuff->cid, RT_CURSOR, (pointer)pCursor))
 	return client->noClientException;
     return BadAlloc;
@@ -2134,12 +2216,13 @@ ProcRenderCreateAnimCursor (ClientPtr client)
 static int
 ProcRenderDispatch (ClientPtr client)
 {
-    int res;
+    int result;
+
     REQUEST(xReq);
 
     /*
-     * Let client fail if we detected that our
-     * RENDER implementation is faulty.
+     * Let the client fail if we are
+     * hiding the RENDER extension.
      */
     
     if (nxagentRenderTrap)
@@ -2156,9 +2239,12 @@ ProcRenderDispatch (ClientPtr client)
          */
 
         nxagentGCTrap = 1;
-	res =  (*ProcRenderVector[stuff->data]) (client);
+
+        result = (*ProcRenderVector[stuff->data]) (client);
+
         nxagentGCTrap = 0;
-        return res;
+
+        return result;
     }
     else
 	return BadRequest;
@@ -2604,7 +2690,7 @@ SProcRenderSetPictureFilter (ClientPtr client)
     swaps(&stuff->nbytes, n);
     return (*ProcRenderVector[stuff->renderReqType]) (client);
 }
-
+    
 static int
 SProcRenderCreateAnimCursor (ClientPtr client)
 {
@@ -2621,10 +2707,36 @@ SProcRenderCreateAnimCursor (ClientPtr client)
 static int
 SProcRenderDispatch (ClientPtr client)
 {
+    int result;
+
     REQUEST(xReq);
     
+    /*
+     * Let the client fail if we are
+     * hiding the RENDER extension.
+     */
+    
+    if (nxagentRenderTrap)
+    {
+        return BadRequest;
+    }
+
     if (stuff->data < RenderNumberRequests)
-	return (*SProcRenderVector[stuff->data]) (client);
+    {
+        /*
+         * Set the nxagentGCTrap flag while
+         * dispatching a render operation to
+         * avoid reentrancy in GCOps.c.
+         */
+
+        nxagentGCTrap = 1;
+
+        result = (*SProcRenderVector[stuff->data]) (client);
+
+        nxagentGCTrap = 0;
+
+        return result;
+    }
     else
 	return BadRequest;
 }
@@ -2932,3 +3044,5 @@ PanoramiXRenderReset (void)
 }
 
 #endif	/* PANORAMIX */
+
+#endif /* #ifdef NXAGENT_UPGRADE */
